SYSADMIN_EXCLUSIVE_POLICIES = [
    ('/arquivos_ibpt', ['POST']),
    ('/arquivos_ibpt/<id>', ['PUT', 'DELETE']),

    ('/ibpt_syncs', ['POST', 'GET']),
    ('/ibpt_syncs/<id>', ['PUT', 'GET', 'DELETE']),
]
SYSADMIN_RESOURCES = [

]
USER_RESOURCES = []

# *ibpt* sysadmin
# ibpt, ibpt_ncm, GET ALL
